--- old/usr/src/cmd/sgs/libld/common/util.c	2013-10-27 14:16:05.076056479 -0400
+++ new/usr/src/cmd/sgs/libld/common/util.c	2013-10-27 14:16:05.088090086 -0400
@@ -301,9 +301,9 @@
 }
 
 /*
- * Determine whether this string, possibly with an associated option, should be
- * translated to an option character.  If so, update the optind and optarg
- * as described for short options in getopt(3c).
+ * Determine whether this string, possibly with an associated option, should
+ * be translated to an option character.  If so, update the optind and optarg
+ * and optopt as described for short options in getopt(3c).
  *
  * entry:
  *	lml - Link map list for debug messages
@@ -329,10 +329,11 @@
 		if (strcmp(arg, opt) == 0) {
 			DBG_CALL(Dbg_args_str2chr(lml, ndx, opt, c));
 			optind += 1;
+			optopt = c;
 			return (c);
 		}
-
-	} else if (strncmp(arg, opt, optsz) == 0) {
+	} else if ((strcmp(arg, opt) == 0) ||
+	    ((arg[optsz] == '=') && strncmp(arg, opt, optsz) == 0)) {
 		/*
 		 * Otherwise, compare the option name, which may be
 		 * concatenated with the option argument.
@@ -345,15 +346,21 @@
 			 * Make sure an optarg is available, and if not return
 			 * a failure to prevent any fall-through to the generic
 			 * getopt() processing.
+			 *
+			 * Since we'll be completely failing this option we
+			 * don't want to update optopt with the translation,
+			 * but also need to set it to _something_.  Setting it
+			 * to the '-' of the argument causes us to behave
+			 * correctly.
 			 */
 			if ((++optind + 1) > argc) {
+				optopt = arg[0];
 				return ('?');
 			}
 			optarg = argv[optind];
 			optind++;
 		} else {
 			/*
-			 * Optarg concatenated to option (no white space).
 			 * GNU option/option argument pairs can be represented
 			 * with a "=" separator.  If this is the case, remove
 			 * the separator.
@@ -362,13 +369,14 @@
 			optind++;
 			if (*optarg == '=') {
 				if (*(++optarg) == '\0')
+					optopt = arg[0];
 					return ('?');
 			}
 		}
 
 		if (cbfunc != NULL)
 			c = (*cbfunc)(c);
-
+		optopt = c;
 		return (c);
 	}
 	return (0);
